@@ -2,11 +2,11 @@ use crate::abi_decoder::ABIDecoder;
 use crate::abi_encoder::ABIEncoder;
 use crate::errors::Error;
 use crate::script::Script;
-use forc::test::{forc_build, BuildCommand};
+use anyhow::Result;
 use fuel_asm::Opcode;
 use fuel_gql_client::client::FuelClient;
 use fuel_tx::{
-    Address, Color, ContractId, Input, Output, Receipt, StorageSlot, Transaction, UtxoId, Witness,
+    Address, AssetId, ContractId, Input, Output, Receipt, StorageSlot, Transaction, UtxoId, Witness,
 };
 use fuel_types::{Bytes32, Immediate12, Salt, Word};
 use fuel_vm::consts::{REG_CGAS, REG_RET, REG_ZERO, VM_TX_MEMORY};
@@ -55,6 +55,7 @@ impl Contract {
     /// Calls an already-deployed contract code.
     /// Note that this is a "generic" call to a contract
     /// and it doesn't, yet, call a specific ABI function in that contract.
+    #[allow(clippy::too_many_arguments)] // We need that many arguments for now
     pub async fn call(
         contract_id: ContractId,
         encoded_selector: Option<Selector>,
@@ -65,21 +66,20 @@ impl Contract {
         byte_price: Word,
         maturity: Word,
         custom_inputs: bool,
+        external_contracts: Option<Vec<ContractId>>,
     ) -> Result<Vec<Receipt>, Error> {
-        // Based on the defined script length,
-        // we set the appropriate data offset.
+        // Based on the defined script length, we set the appropriate data offset.
         let script_len = 16;
         let script_data_offset = VM_TX_MEMORY + Transaction::script_offset() + script_len;
         let script_data_offset = script_data_offset as Immediate12;
 
-        // Script to call the contract.
-        // The offset that points to the `script_data`
-        // is loaded at the register `0x10`. Note that
-        // we're picking `0x10` simply because
-        // it could be any non-reserved register.
-        // Then, we use the Opcode to call a contract: `CALL`
-        // pointing at the register that we loaded the
-        // `script_data` at.
+        // Script to call the contract. The offset that points to the `script_data` is loaded at the
+        // register `0x10`. Note that we're picking `0x10` simply because it could be any
+        // non-reserved register. Then, we use the Opcode to call a contract: `CALL` pointing at the
+        // register that we loaded the `script_data` at.
+
+        // The `iter().collect()` does the Opcode->u8 conversion
+        #[allow(clippy::iter_cloned_collect)]
         let script = vec![
             Opcode::ADDI(0x10, REG_ZERO, script_data_offset),
             Opcode::CALL(0x10, REG_ZERO, 0x10, REG_CGAS),
@@ -109,10 +109,9 @@ impl Contract {
             script_data.extend(e)
         }
 
-        // If the method call takes custom inputs, such as structs or enums,
-        // we need to calculate the `call_data_offset`, which points to
-        // where the data for the custom types start in the transaction.
-        // If it doesn't take any custom inputs, this isn't necessary.
+        // If the method call takes custom inputs, such as structs or enums, we need to calculate
+        // the `call_data_offset`, which points to where the data for the custom types start in the
+        // transaction. If it doesn't take any custom inputs, this isn't necessary.
         if custom_inputs {
             // Offset of the script data relative to the call data
             let call_data_offset = script_data_offset as usize + ContractId::LEN + 2 * WORD_SIZE;
@@ -126,16 +125,38 @@ impl Contract {
             script_data.extend(e)
         }
 
-        // Inputs/outputs
-        let input = Input::contract(
+        let mut inputs: Vec<Input> = vec![];
+        let mut outputs: Vec<Output> = vec![];
+
+        let self_contract_input = Input::contract(
             UtxoId::new(Bytes32::zeroed(), 0),
             Bytes32::zeroed(),
             Bytes32::zeroed(),
             contract_id,
         );
-        // TODO: for now, assume there is only a single input and output, so input_index is 0.
-        // This needs to be changed to support multiple contract IDs.
-        let output = Output::contract(0, Bytes32::zeroed(), Bytes32::zeroed());
+        inputs.push(self_contract_input);
+
+        let self_contract_output = Output::contract(0, Bytes32::zeroed(), Bytes32::zeroed());
+        outputs.push(self_contract_output);
+
+        // Add external contract IDs to Input/Output pair, if applicable.
+        if let Some(external_contract_ids) = external_contracts {
+            for (idx, external_contract_id) in external_contract_ids.iter().enumerate() {
+                let external_contract_input = Input::contract(
+                    UtxoId::new(Bytes32::zeroed(), idx as u8 + 1),
+                    Bytes32::zeroed(),
+                    Bytes32::zeroed(),
+                    *external_contract_id,
+                );
+
+                inputs.push(external_contract_input);
+
+                let external_contract_output =
+                    Output::contract(idx as u8 + 1, Bytes32::zeroed(), Bytes32::zeroed());
+
+                outputs.push(external_contract_output);
+            }
+        }
 
         // Important: this is a temporary workaround, for full context:
         // https://github.com/FuelLabs/fuels-rs/issues/90
@@ -147,13 +168,15 @@ impl Contract {
             UtxoId::new(Bytes32::from(rng.gen::<[u8; 32]>()), 0),
             Address::from(rng.gen::<[u8; 32]>()),
             rng.next_u64(),
-            Color::from([0u8; 32]),
+            AssetId::from([0u8; 32]),
             0,
             0,
             vec![],
             vec![],
         );
 
+        inputs.push(random_coin);
+
         let tx = Transaction::script(
             gas_price,
             gas_limit,
@@ -161,8 +184,8 @@ impl Contract {
             maturity,
             script,
             script_data,
-            vec![input, random_coin],
-            vec![output],
+            inputs,
+            outputs,
             vec![Witness::from(vec![0u8, 0u8])],
         );
 
@@ -215,6 +238,7 @@ impl Contract {
             datatype: PhantomData,
             output_params: output_params.to_vec(),
             custom_inputs,
+            external_contracts: None,
         })
     }
 
@@ -231,29 +255,9 @@ impl Contract {
         }
     }
 
-    /// Compiles a Sway contract
-    pub fn compile_sway_contract(
-        project_path: &str,
-        salt: Salt,
-    ) -> Result<CompiledContract, Error> {
-        let build_command = BuildCommand {
-            debug_outfile: None,
-            minify_json_abi: false,
-            path: Some(project_path.into()),
-            print_finalized_asm: false,
-            print_intermediate_asm: false,
-            binary_outfile: None,
-            offline_mode: false,
-            output_directory: None,
-            silent_mode: true,
-            print_ir: false,
-            use_ir: false,
-        };
-
-        let (raw, _) =
-            forc_build::build(build_command).map_err(|message| Error::CompilationError(message))?;
-
-        Ok(CompiledContract { salt, raw })
+    pub fn load_sway_contract(binary_filepath: &str, salt: Salt) -> Result<CompiledContract> {
+        let bin = std::fs::read(binary_filepath)?;
+        Ok(CompiledContract { raw: bin, salt })
     }
 
     /// Crafts a transaction used to deploy a contract
@@ -309,12 +313,23 @@ pub struct ContractCall<D> {
     pub datatype: PhantomData<D>,
     pub output_params: Vec<ParamType>,
     pub custom_inputs: bool,
+    external_contracts: Option<Vec<ContractId>>,
 }
 
 impl<D> ContractCall<D>
 where
     D: Detokenize,
 {
+    /// Sets external contracts as dependencies to this contract's call.
+    /// Effectively, this will be used to create Input::Contract/Output::Contract
+    /// pairs and set them into the transaction.
+    /// Note that this is a builder method, i.e. use it as a chain:
+    /// `my_contract_instance.my_method(...).set_contracts(&[another_contract_id]).call()`.
+    pub fn set_contracts(mut self, contract_ids: &[ContractId]) -> Self {
+        self.external_contracts = Some(contract_ids.to_vec());
+        self
+    }
+
     /// Call a contract's method. Return a Result<CallResponse, Error>.
     /// The CallResponse structs contains the method's value in its `value`
     /// field as an actual typed value `D` (if your method returns `bool`, it will
@@ -332,6 +347,7 @@ where
             self.byte_price,
             self.maturity,
             self.custom_inputs,
+            self.external_contracts,
         )
         .await?;
 
